@@ -1,5 +1,6 @@
 # Copyright (C) 2018 The Qt Company Ltd.
 # SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+from __future__ import annotations
 
 import importlib
 import os
@@ -12,7 +13,6 @@ import time
 from packaging.version import parse as parse_version
 from pathlib import Path
 from shutil import copytree, rmtree
-from textwrap import dedent
 
 # PYSIDE-1760: Pre-load setuptools modules early to avoid racing conditions.
 #              may be touched (should be avoided anyway, btw.)
@@ -39,7 +39,7 @@ from .platforms.windows_desktop import prepare_packages_win32
 from .qtinfo import QtInfo
 from .utils import (copydir, copyfile, detect_clang,
                     get_numpy_location, get_python_dict,
-                    linux_fix_rpaths_for_library, macos_fix_rpaths_for_library,
+                    linux_fix_rpaths_for_library, macos_fix_rpaths_for_library, parse_modules,
                     platform_cmake_options, remove_tree, run_process,
                     run_process_output, update_env_path, which)
 from . import PYSIDE, PYSIDE_MODULE, SHIBOKEN
@@ -120,7 +120,7 @@ def get_allowed_python_versions():
     pattern = r'Programming Language :: Python :: (\d+)\.(\d+)'
     supported = []
 
-    for line in config.python_version_classifiers:
+    for line in config.classifiers:
         found = re.search(pattern, line)
         if found:
             major = int(found.group(1))
@@ -139,8 +139,11 @@ def check_allowed_python_version():
     supported = get_allowed_python_versions()
     this_py = sys.version_info[:2]
     if this_py not in supported:
-        log.error(f"Unsupported python version detected. Supported versions: {supported}")
-        sys.exit(1)
+        log.warning("*" * 80)
+        log.warning(f"Unsupported Python version detected: {this_py}.")
+        log.warning("The build will probably fail.")
+        log.warning(f"Supported versions: {supported}")
+        log.warning("*" * 80)
 
 
 qt_src_dir = ''
@@ -407,18 +410,18 @@ class PysideBuild(_build, CommandMixin, BuildInfoCollectorMixin):
             for ext in config.get_buildable_extensions():
                 self.build_extension(ext)
 
-            if OPTION["BUILDTESTS"]:
-                # we record the latest successful build and note the
-                # build directory for supporting the tests.
-                timestamp = time.strftime('%Y-%m-%d_%H%M%S')
-                build_history = setup_script_dir / 'build_history'
-                unique_dir = build_history / timestamp
-                unique_dir.mkdir(parents=True)
-                fpath = unique_dir / 'build_dir.txt'
-                with open(fpath, 'w') as f:
-                    print(self.build_dir, file=f)
-                    print(self.build_classifiers, file=f)
-                log.info(f"Created {build_history}")
+            # We always record the history, whether tests are built or not.
+            # Record the latest successful build and note the
+            # build directory for supporting the tests or other tools.
+            timestamp = time.strftime('%Y-%m-%d_%H%M%S')
+            build_history = setup_script_dir / 'build_history'
+            unique_dir = build_history / timestamp
+            unique_dir.mkdir(parents=True)
+            fpath = unique_dir / 'build_dir.txt'
+            with open(fpath, 'w') as f:
+                print(self.build_dir, file=f)
+                print(self.build_classifiers, file=f)
+            log.info(f"Created {build_history}")
 
         if not OPTION["SKIP_PACKAGING"]:
             # Build patchelf if needed
@@ -465,6 +468,7 @@ class PysideBuild(_build, CommandMixin, BuildInfoCollectorMixin):
                 except Exception as e:
                     log.warning(f'problem renaming "{self.st_build_dir}"')
                     log.warning(f'ignored error: {type(e).__name__}: {e}')
+
         else:
             log.info("Skipped preparing and building packages.")
         log.info(f"--- Build completed ({elapsed()}s)")
@@ -485,27 +489,13 @@ class PysideBuild(_build, CommandMixin, BuildInfoCollectorMixin):
         log.info(f"Make generator: {self.make_generator}")
         log.info(f"Make jobs:      {OPTION['JOBS']}")
         log.info("-" * 3)
-        log.info(f"setup.py directory:      {self.script_dir}")
-        log.info(f"Build scripts directory: {build_scripts_dir}")
-        log.info(f"Sources directory:       {self.sources_dir}")
-        log.info(dedent(f"""
-        Building {config.package_name()} will create and touch directories
-          in the following order:
-            make build directory ->
-            make install directory ->
-            setuptools build directory ->
-            setuptools install directory
-              (usually path-installed-python/lib/python*/site-packages/*)
-         """))
+        log.info(f"setup.py directory:           {self.script_dir}")
+        log.info(f"Build scripts directory:      {build_scripts_dir}")
+        log.info(f"Sources directory:            {self.sources_dir}")
         log.info(f"make build directory:         {self.build_dir}")
         log.info(f"make install directory:       {self.install_dir}")
         log.info(f"setuptools build directory:   {self.st_build_dir}")
         log.info(f"setuptools install directory: {setuptools_install_prefix}")
-        log.info(dedent(f"""
-        make-installed site-packages directory: {self.site_packages_dir}
-         (only relevant for copying files from 'make install directory'
-                                          to   'setuptools build directory'
-         """))
         log.info("-" * 3)
         log.info(f"Python executable: {self.py_executable}")
         log.info(f"Python includes:   {self.py_include_dir}")
@@ -657,24 +647,11 @@ class PysideBuild(_build, CommandMixin, BuildInfoCollectorMixin):
                          f"Path given: {config_dir}")
 
         if OPTION["MODULE_SUBSET"]:
-            module_sub_set = ''
-            for m in OPTION["MODULE_SUBSET"].split(','):
-                if m.startswith('Qt'):
-                    m = m[2:]
-                if module_sub_set:
-                    module_sub_set += ';'
-                module_sub_set += m
-            cmake_cmd.append(f"-DMODULES={module_sub_set}")
+            cmake_cmd.append(f"-DMODULES={parse_modules(OPTION['MODULE_SUBSET'])}")
 
         if OPTION["SKIP_MODULES"]:
-            skip_modules = ''
-            for m in OPTION["SKIP_MODULES"].split(','):
-                if m.startswith('Qt'):
-                    m = m[2:]
-                if skip_modules:
-                    skip_modules += ';'
-                skip_modules += m
-            cmake_cmd.append(f"-DSKIP_MODULES={skip_modules}")
+            cmake_cmd.append(f"-DSKIP_MODULES={parse_modules(OPTION['SKIP_MODULES'])}")
+
         # Add source location for generating documentation
         cmake_src_dir = OPTION["QT_SRC"] if OPTION["QT_SRC"] else qt_src_dir
         if cmake_src_dir:
@@ -710,21 +687,28 @@ class PysideBuild(_build, CommandMixin, BuildInfoCollectorMixin):
             if OPTION['NO_OVERRIDE_OPTIMIZATION_FLAGS']:
                 cmake_cmd.append("-DQFP_NO_OVERRIDE_OPTIMIZATION_FLAGS=1")
 
-        if OPTION["LIMITED_API"] == "yes":
-            cmake_cmd.append("-DFORCE_LIMITED_API=yes")
-        elif OPTION["LIMITED_API"] == "no":
-            cmake_cmd.append("-DFORCE_LIMITED_API=no")
-        elif not OPTION["LIMITED_API"]:
+        if not OPTION["LIMITED_API"]:
             if sys.platform == 'win32' and self.debug:
                 cmake_cmd.append("-DFORCE_LIMITED_API=no")
         else:
-            raise SetupError("option limited-api must be 'yes' or 'no' "
-                             "(default yes if applicable, i.e. Python "
-                             "version >= 3.9 and release build if on Windows)")
+            if OPTION["LIMITED_API"].lower() in ("yes", "y", "1", "true"):
+                cmake_cmd.append("-DFORCE_LIMITED_API=yes")
+            elif OPTION["LIMITED_API"].lower() in ("no", "n", "0", "false"):
+                cmake_cmd.append("-DFORCE_LIMITED_API=no")
+            else:
+                raise SetupError(
+                    "Option '--limited-api' must be 'yes' or 'no'."
+                    f"Default is yes if Python version >= {get_allowed_python_versions()[0]} "
+                    "and Release build on Windows"
+                )
 
         if OPTION["DISABLE_PYI"]:
             cmake_cmd.append("-DDISABLE_PYI=yes")
 
+        if OPTION["UNOPTIMIZE"]:
+            value = OPTION["UNOPTIMIZE"]
+            cmake_cmd.append(f"-DSHIBOKEN_UNOPTIMIZE={value}")
+
         if OPTION["LOG_LEVEL"] == LogLevel.VERBOSE:
             cmake_cmd.append("-DCMAKE_VERBOSE_MAKEFILE:BOOL=ON")
         else:
@@ -736,12 +720,12 @@ class PysideBuild(_build, CommandMixin, BuildInfoCollectorMixin):
             cmake_cmd.append(f"-DCMAKE_CXX_COMPILER_LAUNCHER={compiler_launcher}")
 
         if OPTION["SANITIZE_ADDRESS"]:
+            cmake_cmd.append("-DSANITIZE_ADDRESS=ON")
+        if OPTION["SANITIZE_THREAD"]:
             # Some simple sanity checking. Only use at your own risk.
-            if (sys.platform.startswith('linux')
-                    or sys.platform.startswith('darwin')):
-                cmake_cmd.append("-DSANITIZE_ADDRESS=ON")
-            else:
-                raise SetupError("Address sanitizer can only be used on Linux and macOS.")
+            if sys.platform == "win32" and not self.is_cross_compile:
+                self.warn("Thread sanitizer may not be supported yet.")
+            cmake_cmd.append("-DSANITIZE_THREAD=ON")
 
         if extension.lower() == PYSIDE:
             pyside_qt_conf_prefix = ''
@@ -855,6 +839,9 @@ class PysideBuild(_build, CommandMixin, BuildInfoCollectorMixin):
             # cross-compiling pyside.
             cmake_cmd.append(f"-DQFP_SHIBOKEN_TARGET_PATH={self.install_dir}")
 
+        if OPTION["SKIP_MYPY_TEST"]:
+            cmake_cmd.append("-DSKIP_MYPY_TEST=1")
+
         if self.cmake_toolchain_file:
             cmake_cmd.append(f"-DCMAKE_TOOLCHAIN_FILE={self.cmake_toolchain_file}")
 
@@ -874,13 +861,6 @@ class PysideBuild(_build, CommandMixin, BuildInfoCollectorMixin):
         if run_process(cmd_make) != 0:
             raise SetupError(f"Error compiling {extension}")
 
-        if sys.version_info == (3, 6) and sys.platform == "darwin":
-            # Python 3.6 has a Sphinx problem because of docutils 0.17 .
-            # Instead of pinning v0.16, setting the default encoding fixes that.
-            # Since other platforms are not affected, we restrict this to macOS.
-            if "UTF-8" not in os.environ.get("LC_ALL", ""):
-                os.environ["LC_ALL"] = "en_US.UTF-8"
-
         if OPTION["BUILD_DOCS"]:
             if extension.lower() == SHIBOKEN:
                 found = importlib.util.find_spec("sphinx")
@@ -1068,6 +1048,11 @@ class PysideBuild(_build, CommandMixin, BuildInfoCollectorMixin):
 
         if clang_lib_path.exists():
             basename = clang_lib_path.name
+            # In case of static libclang we don't need the lib file inside the wheel
+            if '.a' == clang_lib_path.suffix:
+                log.info("Skip copying libclang archive to the package.")
+                return
+
             log.info(f"Copying libclang shared library {clang_lib_path} to the package "
                      f"folder as {basename}.")
             destination_path = destination_dir / basename
@@ -1244,10 +1229,6 @@ class PysideBaseDocs(Command, CommandMixin):
     user_options = CommandMixin.mixin_user_options
 
     def __init__(self, *args, **kwargs):
-        if args[0].commands[0] == "build_rst_docs":
-            args[0].commands[0] = "build_base_docs"
-            log.warning("'build_rst_docs' is deprecated and will be removed. "
-                        "Please use 'build_base_docs' instead.")
         self.command_name = "build_base_docs"
         Command.__init__(self, *args, **kwargs)
         CommandMixin.__init__(self)
@@ -1324,21 +1305,42 @@ class PysideBaseDocs(Command, CommandMixin):
                 raise SetupError(f"Error running CMake for {self.doc_dir}")
 
             if self.name == PYSIDE:
+                def run_script(script_path, args=None):
+                    cmd = [sys.executable, os.fspath(script_path)]
+                    if args:
+                        cmd.extend(args)
+                    if run_process(cmd) != 0:
+                        raise SetupError(f"Error running {script_path}")
+
                 self.sphinx_src = self.out_dir / "base"
+                # Generates the .rst files from the examples
                 example_gallery = config.setup_script_dir / "tools" / "example_gallery" / "main.py"
-                assert example_gallery.is_file()
-                example_gallery_cmd = [sys.executable, os.fspath(example_gallery)]
-                if OPTION["LOG_LEVEL"] == LogLevel.QUIET:
-                    example_gallery_cmd.append("--quiet")
-                qt_src_dir = OPTION['QT_SRC']
-                if qt_src_dir:
-                    example_gallery_cmd.extend(["--qt-src-dir", qt_src_dir])
-                if run_process(example_gallery_cmd) != 0:
-                    raise SetupError(f"Error running example gallery for {self.doc_dir}")
+                if example_gallery.is_file():
+                    example_gallery_args = []
+                    if OPTION["LOG_LEVEL"] == LogLevel.QUIET:
+                        example_gallery_args.append("--quiet")
+                    qt_src_dir = OPTION['QT_SRC']
+                    if qt_src_dir:
+                        example_gallery_args.extend(["--qt-src-dir", qt_src_dir])
+                    run_script(example_gallery, example_gallery_args)
+                else:
+                    log.warning("Example gallery script for generating .rst for examples"
+                                f"not found: {example_gallery}")
+
+                # Generates the .rst files from the release notes
+                release_notes = config.setup_script_dir / "tools" / "release_notes" / "main.py"
+                if release_notes.is_file():
+                    release_notes_args = []
+                    if OPTION["LOG_LEVEL"] != LogLevel.QUIET:
+                        release_notes_args.append("--verbose")
+                    run_script(release_notes, release_notes_args)
+                else:
+                    log.warning("Release notes script for generating .rst for release notes"
+                                f"not found: {release_notes}")
             elif self.name == SHIBOKEN:
                 self.sphinx_src = self.out_dir
 
-            sphinx_cmd = ["sphinx-build", "-b", "html", "-j", "auto", "-c",
+            sphinx_cmd = ["sphinx-build", "-b", "html", "-j", "auto", "-n", "-c",
                           str(self.sphinx_src), str(self.doc_dir),
                           str(self.out_dir)]
             if run_process(sphinx_cmd) != 0:
@@ -1360,8 +1362,6 @@ cmd_class_dict = {
     'install': PysideInstall,
     'install_lib': PysideInstallLib,
     'build_base_docs': PysideBaseDocs,
-    # TODO: Remove build_rst_docs in the next version, see PYSIDE-2504
-    'build_rst_docs': PysideBaseDocs,
 }
 if wheel_module_exists:
     pyside_bdist_wheel = get_bdist_wheel_override()
