@@ -9,12 +9,14 @@
 #include "pysidestaticstrings.h"
 #include "qobjectconnect.h"
 #include "signalmanager.h"
+#include "pysideqenum.h"
 
 #include <autodecref.h>
 #include <helper.h>
 #include <pep384ext.h>
 #include <sbkconverter.h>
 #include <sbkenum.h>
+#include <sbkerrors.h>
 #include <sbkstaticstrings.h>
 #include <sbkstring.h>
 #include <sbktypefactory.h>
@@ -97,7 +99,7 @@ static bool connection_Check(PyObject *o)
     static QByteArray typeName = QByteArrayLiteral("PySide")
         + QByteArray::number(QT_VERSION_MAJOR)
         + QByteArrayLiteral(".QtCore.QMetaObject.Connection");
-    return std::strcmp(o->ob_type->tp_name, typeName.constData()) == 0;
+    return std::strcmp(PepType_GetFullyQualifiedNameStr(Py_TYPE(o)), typeName.constData()) == 0;
 }
 
 static std::optional<QByteArrayList> parseArgumentNames(PyObject *argArguments)
@@ -448,7 +450,7 @@ static FunctionArgumentsResult extractFunctionArgumentsFromSlot(PyObject *slot)
         // it being actually being that.
         if (ret.objCode == nullptr)
             ret.function = nullptr;
-    } else if (std::strcmp(Py_TYPE(slot)->tp_name, "compiled_function") == 0) {
+    } else if (std::strcmp(PepType_GetFullyQualifiedNameStr(Py_TYPE(slot)), "compiled_function") == 0) {
         ret.isMethod = false;
         ret.function = slot;
 
@@ -539,7 +541,8 @@ static PyObject *signalInstanceConnect(PyObject *self, PyObject *args, PyObject
         return nullptr;
 
     Qt::ConnectionType connectionType = Qt::AutoConnection;
-    if (type != nullptr && qstrcmp(Py_TYPE(type)->tp_name, "ConnectionType") == 0) {
+    if (type != nullptr
+        && qstrcmp(PepType_GetFullyQualifiedNameStr(Py_TYPE(type)), "ConnectionType") == 0) {
         static SbkConverter *connectionTypeConv =
             Shiboken::Conversions::getConverter("Qt::ConnectionType");
         Q_ASSERT(connectionTypeConv);
@@ -666,13 +669,9 @@ static PyObject *signalInstanceGetItem(PyObject *self, PyObject *key)
 static inline void warnDisconnectFailed(PyObject *aSlot, const QByteArray &signature)
 {
     if (PyErr_Occurred() != nullptr) { // avoid "%S" invoking str() when an error is set.
-        PyObject *exc{};
-        PyObject *inst{};
-        PyObject *tb{};
-        PyErr_Fetch(&exc, &inst, &tb);
+        Shiboken::Errors::Stash errorStash;
         PyErr_WarnFormat(PyExc_RuntimeWarning, 0, "Failed to disconnect (%s) from signal \"%s\".",
                          Py_TYPE(aSlot)->tp_name, signature.constData());
-        PyErr_Restore(exc, inst, tb);
     } else {
         PyErr_WarnFormat(PyExc_RuntimeWarning, 0, "Failed to disconnect (%S) from signal \"%s\".",
                          aSlot, signature.constData());
@@ -913,21 +912,21 @@ void init(PyObject *module)
         return;
     auto *obMetaSignalType = reinterpret_cast<PyObject *>(metaSignalType);
     Py_INCREF(obMetaSignalType);
-    PyModule_AddObject(module, "MetaSignal", obMetaSignalType);
+    PepModule_AddType(module, metaSignalType);
 
     auto *signalType = PySideSignal_TypeF();
     if (InitSignatureStrings(signalType, Signal_SignatureStrings) < 0)
         return;
     auto *obSignalType = reinterpret_cast<PyObject *>(signalType);
     Py_INCREF(obSignalType);
-    PyModule_AddObject(module, "Signal", obSignalType);
+    PepModule_AddType(module, signalType);
 
     auto *signalInstanceType = PySideSignalInstance_TypeF();
     if (InitSignatureStrings(signalInstanceType, SignalInstance_SignatureStrings) < 0)
         return;
     auto *obSignalInstanceType = reinterpret_cast<PyObject *>(signalInstanceType);
     Py_INCREF(obSignalInstanceType);
-    PyModule_AddObject(module, "SignalInstance", obSignalInstanceType);
+    PepModule_AddType(module, signalInstanceType);
 }
 
 bool checkType(PyObject *pyObj)
@@ -991,23 +990,6 @@ void updateSourceObject(PyObject *source)
         return;
 }
 
-// PYSIDE-2840: For an enum registered in Qt, return the C++ name.
-// Ignore flags here; their underlying enums are of Python type flags anyways.
-static QByteArray getQtEnumTypeName(PyTypeObject *type)
-{
-    if (!Shiboken::Enum::checkType(type))
-        return {};
-
-    Shiboken::AutoDecRef qualName(PyObject_GetAttr(reinterpret_cast<PyObject *>(type),
-                                                   Shiboken::PyMagicName::qualname()));
-    QByteArray result = Shiboken::String::toCString(qualName.object());
-    result.replace(".", "::");
-
-    const auto metaType = QMetaType::fromName(result);
-    return metaType.isValid() && metaType.flags().testFlag(QMetaType::IsEnumeration)
-        ? result : QByteArray{};
-}
-
 QByteArray getTypeName(PyObject *obType)
 {
     if (PyType_Check(obType)) {
@@ -1027,7 +1009,7 @@ QByteArray getTypeName(PyObject *obType)
             return QByteArrayLiteral("QVariantList");
         if (type == &PyDict_Type)
             return QByteArrayLiteral("QVariantMap");
-        QByteArray enumName = getQtEnumTypeName(type);
+        QByteArray enumName = PySide::QEnum::getTypeName(type);
         return enumName.isEmpty() ? "PyObject"_ba : enumName;
     }
     if (obType == Py_None) // Must be checked before as Shiboken::String::check accepts Py_None
