@@ -7,6 +7,7 @@
 
 #include <autodecref.h>
 #include <basewrapper.h>
+#include <sbkpep.h>
 #include <sbkstaticstrings.h>
 #include <sbkstring.h>
 #include <sbktypefactory.h>
@@ -177,11 +178,13 @@ static const char *Slot_SignatureStrings[] = {
 
 void init(PyObject *module)
 {
-    if (InitSignatureStrings(PySideSlot_TypeF(), Slot_SignatureStrings) < 0)
+    auto *slotType = PySideSlot_TypeF();
+    if (InitSignatureStrings(slotType, Slot_SignatureStrings) < 0)
         return;
 
-    Py_INCREF(PySideSlot_TypeF());
-    PyModule_AddObject(module, "Slot", reinterpret_cast<PyObject *>(PySideSlot_TypeF()));
+    auto *obSlotType = reinterpret_cast<PyObject *>(slotType);
+    Py_INCREF(obSlotType);
+    PepModule_AddType(module, slotType);
 }
 
 } // namespace PySide::Slot
